/**
 * Copyright(c) 1996 DTA Consulting (http://www.dtai.com)
 *
 *                        All rights reserved
 *
 * Permission to use, copy, modify and distribute this material for
 * any purpose and without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all
 * copies, and that the name of DTA Consulting not be used in
 * advertising or publicity pertaining to this material without the
 * specific, prior written permission of an authorized representative of
 * DTA Consulting.
 *
 * DTA CONSULTING MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES,
 * EXPRESS OR IMPLIED, WITH RESPECT TO THE SOFTWARE, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR ANY PARTICULAR PURPOSE, AND THE WARRANTY AGAINST
 * INFRINGEMENT OF PATENTS OR OTHER INTELLECTUAL PROPERTY RIGHTS.  THE
 * SOFTWARE IS PROVIDED "AS IS", AND IN NO EVENT SHALL DTA CONSULTING OR
 * ANY OF ITS AFFILIATES BE LIABLE FOR ANY DAMAGES, INCLUDING ANY
 * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES RELATING
 * TO THE SOFTWARE.
 */

//package COM.dtai.net;

import java.net.*;
import java.io.*;

public class NetPort
    extends Thread
    implements DataInput, DataOutput
{
    private Socket socket;
    private DataOutputStream outstream;
    private DataInputStream instream;

    protected NetPort( Socket socket ) throws IOException
    {
        this.socket = socket;

        instream = new DataInputStream( socket.getInputStream() );
        outstream = new DataOutputStream( socket.getOutputStream() );
    }

/**
 * This is normally invoked by calling "start()" to run this in its own thread.
 * It loops forever and calls readInput().  If error, calls error().
 */
    public void run () {

        while ( true ) {
            try {
            	readInput ();
            }
            catch ( IOException ioerror ) {
	            error( "Error reading input stream." );
	            break;
            }
        }
    }

    public void close() {
        try {
            instream.close();
            outstream.close();
            socket.close();
        }
        catch( IOException ioerror ) {
            error( "Error closing input stream." );
        }
    }

    protected void finalize() {
        close();
    }

    public DataInputStream getInputStream () {
        return ( instream );
    }

    public DataOutputStream getOutputStream () {
        return ( outstream );
    }

/**
 * Generally called by run if an error is caught.  It does nothing unless
 * it is extended in a subclass.
 * @param    message     The error message.
 */
    public synchronized void error( String message ) {
        // subclass to extend
    }

    private synchronized void doNothing() {
        try {
            wait();
        }
        catch ( InterruptedException ie ) {
        }
    }

/**
 * This function should be extended in a subclass.  It is called by "run()"
 * (normally if a thread is started with "start()") and is continually recalled
 * in a loop.
 */
    protected void readInput() throws IOException {
        doNothing();
    }

// implement DataOutput
///////////////////////////////////////////////////////////////////////////////

    public void flush() throws IOException { // not part of DataOutput interface, but necessary for
                          // DataOutputStream class
        outstream.flush();
    }

    public void write( int byteval ) throws IOException {
        outstream.write( byteval );
    }

    public void write( byte b[] ) throws IOException {
        outstream.write( b );
    }

    public void write( byte b[], int off, int len ) throws IOException {
        outstream.write( b, off, len );
    }

    public void writeBoolean( boolean v ) throws IOException {
        outstream.writeBoolean( v );
    }

    public void writeByte( int v ) throws IOException {
        outstream.writeByte( v );
    }

    public void writeBytes( String s ) throws IOException {
        outstream.writeBytes( s );
    }

    public void writeChar( int v ) throws IOException {
        outstream.writeChar( v );
    }

    public void writeChars( String s ) throws IOException {
        outstream.writeChars( s );
    }

    public void writeDouble( double v ) throws IOException {
        outstream.writeDouble( v );
    }

    public void writeFloat( float v ) throws IOException {
        outstream.writeFloat( v );
    }

    public void writeInt( int v ) throws IOException {
        outstream.writeInt( v );
    }

    public void writeLong( long v ) throws IOException {
        outstream.writeLong( v );
    }

    public void writeShort( int v ) throws IOException {
        outstream.writeShort( v );
    }

    public void writeUTF( String str ) throws IOException {
        outstream.writeUTF( str );
    }

    public void readFully(byte b[]) throws IOException {
        instream.readFully( b );
    }

    public void readFully(byte b[], int off, int len) throws IOException {
        instream.readFully( b, off, len );
    }

    public int skipBytes(int n) throws IOException {
        return instream.skipBytes( n );
    }

    public boolean readBoolean() throws IOException {
        return instream.readBoolean();
    }

    public byte readByte() throws IOException {
        return instream.readByte();
    }

    public int readUnsignedByte() throws IOException {
        return instream.readUnsignedByte();
    }

    public short readShort() throws IOException {
        return instream.readShort();
    }

    public int readUnsignedShort() throws IOException {
        return instream.readUnsignedShort();
    }

    public char readChar() throws IOException {
        return instream.readChar();
    }

    public int readInt() throws IOException {
        return instream.readInt();
    }

    public long readLong() throws IOException {
        return instream.readLong();
    }

    public float readFloat() throws IOException {
        return instream.readFloat();
    }

    public double readDouble() throws IOException {
        return instream.readDouble();
    }

    public String readLine() throws IOException {
        return instream.readLine();
    }

    public String readUTF() throws IOException {
        return instream.readUTF();
    }
}
